"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebpackProgressOraPlugin = exports.ProgressOraPlugin = void 0;
var chalk = require('chalk');
var webpack = require('webpack');
var ora = require('ora');
var ProgressBar = require('./ProgressBar');
var default_pattern = chalk.bold('[') + ':percent:%' + chalk.bold('] ') + chalk.cyan.bold(':text:');
function ProgressOraPlugin(options) {
    if (options === void 0) { options = {}; }
    options.pattern = options.pattern || default_pattern;
    options.pattern_no_stderr = options.pattern_no_stderr || chalk.bold('█');
    options.update_render = options.update_render || false;
    options.clear = options.clear || false;
    options.clear_on_update = options.clear_on_update || false;
    var pb = new ProgressBar('Generate...', 20, options.pattern_no_stderr);
    var stderr_check = false;
    var stream = options.stream || process.stderr;
    var enabled = stream && stream.isTTY;
    if (!enabled) {
        if (!options.stderr_check) {
            return function () { };
        }
        stderr_check = true;
    }
    if (options.clear && !stderr_check) {
        stream.write('\x1Bc');
    }
    else {
        stream.write('\n');
    }
    options.text = options.pattern.replace(/\:percent\:/, '0').replace(/\:text\:/, 'build start');
    var spinner = !stderr_check ? ora(options).start() : null;
    var isRunning = false;
    var startTime = 0;
    return new webpack.ProgressPlugin(function (percent, msg) {
        var newPercent = Math.ceil(percent * 100);
        if (!stderr_check) {
            if (options.clear_on_update) {
                stream.write('\x1Bc');
            }
            if (options.pattern)
                spinner.text = options.pattern.replace(/\:percent\:/, newPercent.toString()).replace(/\:text\:/, msg);
            if (options.update_render) {
                spinner.render();
            }
        }
        else {
            pb.render({
                percent: (newPercent / 100).toFixed(4),
                completed: newPercent,
                total: 100,
            });
        }
        if (!isRunning) {
            isRunning = true;
            startTime = (new Date()).getTime();
        }
        else if (percent >= 1) {
            var now = (new Date()).getTime();
            var buildTime = (now - startTime) / 1000 + 's';
            if (stderr_check) {
                stream.write('\n' + chalk.green('✓') + chalk.green.bold(' Build completed in ' + buildTime + '\n\n'));
            }
            else {
                spinner.stop();
                if (options.clear) {
                    stream.write('\x1Bc');
                }
                spinner.succeed(chalk.green.bold('Build completed in ' + buildTime + '\n\n'));
            }
            isRunning = false;
        }
    });
}
exports.ProgressOraPlugin = ProgressOraPlugin;
;
var WebpackProgressOraPlugin = /** @class */ (function () {
    function WebpackProgressOraPlugin(options) {
        if (options === void 0) { options = {}; }
        return ProgressOraPlugin(options);
    }
    return WebpackProgressOraPlugin;
}());
exports.WebpackProgressOraPlugin = WebpackProgressOraPlugin;
exports.default = ProgressOraPlugin;
